#!/bin/bash

module load gcc-5
source /scratch/app/modulos/intel-psxe-2016.2.062.sh

mpiicc -qopenmp xthi.c -o xthi-mpi
gcc -fopenmp xthi-nested-omp.c -o xthi-nested-omp
gcc -fopenmp xthi-omp.c -o xthi-omp
